/*  FreeNote for Sharp SLA300, B500, C7x0, C860 Linux PDA
	Copyright (C) 2003-2005 Joe Kanemori.<kanemori@ymg.urban.ne.jp>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* 
2005/02/27 FreeNote 1.11.10pre
EPDF̏o͌`ꕔύX
EC|[g̃oOfix

2004/10/17 FreeNote 1.10.0[X
2003/12/04 ver 1.5.3pre
EnewpageƂAÕtH_ێB

2003/09/15 ver 1.4.0
EftHg̃t@C莩B

2003/08/15 FreeNote 1.2.1J
Eۑ̃oOC
E_CAO̎
EPNGt@Cւ̏o

2003/08/15 FreeNote 1.2J
EIvVǉ
EXN[KCh
EFreet@C֘At
EACR̕ύX

2003/08/05 FreeNote 1.1.1preJ
ENɕԂێ
E`惂[hؑւɏS\
EۑԒZk
EView[h̋ύX
Ej[̌ڂύX
*/
#include <qdir.h>
#include <qdatetime.h>
#include <qpe/qpemenubar.h>
#include "fnfiledialog.h"

FNFileSelector::FNFileSelector(const QString& dirPath, const QString& mimefilter, QWidget* parent, const char* name)
:FNFileSelectorBase(parent, name, 0), _dir(dirPath), _filter(mimefilter), _isblock(false)
{
}

FNFileSelector::~FNFileSelector()
{
}

void FNFileSelector::setCurrentFile(const QString& fName)
{
	QFileInfo fi(fName);
	pathChanged(fi.dirPath());
	bool isHit = false;
	if (NULL != ListFile->selectedItem()) {
		while (1) {
			if (ListFile->selectedItem()->text(0) == fi.fileName()) {
				isHit = true;
				break;
			}
			QListViewItem* itm = ListFile->selectedItem()->itemBelow();
			if (NULL == itm) {
				break;
			} else {
				ListFile->setSelected(ListFile->selectedItem(), false);
				ListFile->setSelected(itm, true);
			}
		}
		if (!isHit) {
			ListFile->setSelected(ListFile->selectedItem(), false);
			_selected = fi;
			emit fileSelected(fi);
		}
	}
}

const QFileInfo* FNFileSelector::selected()
{
	if (NULL == ListFile->selectedItem()) {
		return NULL;
	} else {
		return &_selected;
	}
}

void FNFileSelector::selectionChanged(QListViewItem* itm)
{
	if (NULL != itm) {
		_selected = QFileInfo(CBxDir->currentText() + "/" + itm->text(0));
		emit fileSelected(_selected);
	}
}

void FNFileSelector::pathChanged(const QDir& d)
{
	emit dirOpened(d);
	rebuildList(d);
}

void FNFileSelector::pathChanged(const QString& s)
{
	pathChanged(QDir(s));
}

void FNFileSelector::rebuildList(QDir dir)
{
	if (_isblock) {
		return;
	}
	_isblock = true;
	if (!dir.exists()) {
		dir = QDir("/");
	}
	//files_.clear();
	//dirs_.clear();
	ListFile->clear();
	CBxDir->clear();

	dir.setFilter(QDir::Dirs | QDir::Readable);
	dir.setSorting(QDir::Name);
	QFileInfoList* list = (QFileInfoList*)dir.entryInfoList();
	if (NULL == list) {
		_isblock = false;
		return;
	}
	if (!list->isEmpty()) {
	    QFileInfoListIterator dt(*list);
		QFileInfo *fi;
		while (NULL != (fi = dt.current())) {
			// ʃfBNg𑖍(ċA)
			CBxDir->insertItem(fi->absFilePath());
			++dt;
		}
	}
	dir.setFilter(QDir::Files | QDir::NoSymLinks | QDir::Readable);
	dir.setSorting(QDir::Name);
	list = (QFileInfoList*)dir.entryInfoList(_filter);
	if (!list->isEmpty()) {
	    QFileInfoListIterator dt(*list);
		QFileInfo *fi;
		while (NULL != (fi = dt.current())) {
			// ʃfBNg𑖍(ċA)
			static char cvbuf[1024];
			const QDate& d = fi->lastModified().date();
			const QTime& t = fi->lastModified().time();
			sprintf(cvbuf, "%04d/%02d/%02d %02d:%02d:%02d ", d.year(), d.month(), d.day(), t.hour(), t.minute(), t.second());
			ListFile->insertItem(new QListViewItem(ListFile, fi->fileName(), cvbuf, QString::number(fi->size())));
			++dt;
		}
	    ListFile->setSorting(1, false);
		ListFile->sort();
		//ListFile->setSelected(ListFile->firstChild(), true);
	}
	_isblock = false;
}


const QString FNFileSelector::currentDir() const
{
	return CBxDir->currentText();
}

FNFileDialog::FNFileDialog(QWidget* parent, const char* filter, const char* title, bool isOpenDialog)
: QDialog(parent, 0, true, 0), _info("")
{
	this->setCaption(title);
	QVBoxLayout* _vbox = new QVBoxLayout(this);
    _vbox->addWidget(_menubar = new QPEMenuBar(this));
    _menubar->insertItem("/     ", this, SLOT(changeRoot()));
    _menubar->insertItem("/mnt  ", this, SLOT(changeMnt()));
    _menubar->insertItem("/home ", this, SLOT(changeHome()));

	_vbox->addWidget(_edtFName = new QLineEdit(this));
	if (isOpenDialog) {
		_edtFName->hide(); 
	}
	_vbox->addWidget(_selector = new FNFileSelector("~/", filter, this, "fileselector"));
	_vbox->activate();
	connect(_selector, SIGNAL(fileSelected(const QFileInfo&)), this, SLOT(fileSelected(const QFileInfo&)));
	connect(_selector, SIGNAL(dirOpened(const QDir&)), this, SLOT(dirOpened(const QDir&)));
}

FNFileDialog::~FNFileDialog()
{
	//delete _edtFName;
	//delete _vbox;
	//delete _selector;
}

void FNFileDialog::changeRoot()
{
	_selector->pathChanged(QDir("/"));
}

void FNFileDialog::changeMnt()
{
	_selector->pathChanged(QDir("/mnt"));
}

void FNFileDialog::changeHome()
{
	_selector->pathChanged(QDir("/home"));
}

void FNFileDialog::dirOpened(const QDir& dir)
{
	_info.setFile(dir, "");
}

void FNFileDialog::fileSelected(const QFileInfo& file)
{
	if (_info.absFilePath() == file.absFilePath()) {
		accept();
	} else {
		_info = file;
		_edtFName->setText(_info.fileName());
	}
}

void FNFileDialog::setCurrentFile(const QFileInfo& info)
{
	_info = info;
	if (_info.isDir() || _info.baseName() == "") {
		QDir dir = _info.dir();
		QDateTime dtime = QDateTime::currentDateTime();
		QDate dt = dtime.date();
		QTime tm = dtime.time();
		char buf[11];
		sprintf(buf, "%02d%02d%02d%02d%02d", dt.year() % 100, dt.month(), dt.day(), tm.hour(), tm.minute());
		if (_info.dir().exists()) {
			_selector->pathChanged(dir);
			if (NULL != _selector->selected()) {
				_info.setFile(_info.dir(), _selector->selected()->fileName());
			} else {
				_info.setFile(_info.dir(), buf);
			}
		} else {
			_info.setFile("~/" + QString(buf));
		}
		_edtFName->setText(buf);
		_selector->setCurrentFile(_info.absFilePath());
	} else {
		_edtFName->setText(_info.fileName());
		_selector->setCurrentFile(_info.absFilePath());
	}
}

const QFileInfo& FNFileDialog::currentFile()
{
	QDir dir = _selector->currentDir();
	_info.setFile(dir, _edtFName->text());
	return _info;
}


void FNFileDialog::resizeEvent(QResizeEvent*)
{
	if (_edtFName->isVisible()) {
		_edtFName->setGeometry(QRect(0, _menubar->height(), width(), _edtFName->height()));
		_selector->setGeometry(QRect(0, _menubar->height() + _edtFName->height(), width(), height() - _menubar->height() - _edtFName->height()));
	} else {
		_selector->setGeometry(QRect(0, _menubar->height(), width(), height() - _menubar->height()));
	}
	//_vbox->setGeometry(QRect(0, _menubar->height(), width(), height() - _menubar->height()));
}
