/*  FreeNote for Sharp SLA300, B500, C7x0, C860, C3000 Linux PDA
	Copyright (C) 2003-2005 Joe Kanemori.<kanemori@ymg.urban.ne.jp>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
2005/06/04 FreeNote 1.11.12pre
EObh̘gۂ̑I͈͂傫\悤ɕύXB
E͈͑IAObhAKChArJ[̃JX^}CY\ɁB
EJ[sbJ[[h̒ǉ

2005/06/02 FreeNote 1.11.11Apre
EeLXgړg̕sC
ErObĥQ{ŕ\悤ɕύX
ES͈͎̔w莞ɁAyItăe|Ă͈͊m肷悤ɕύX

2005/02/27 FreeNote 1.11.10pre
EPDF̏o͌`ꕔύX
EC|[g̃oOfix

2004/10/17 FreeNote 1.10.0[X
2004/02/12 ver 1.7.1pre
EtHgdl̕ύX
EeLXg̍
EeLXg{bNX̑@\

2004/02/10 ver 1.7.0pre
E

2003/12/22 ver 1.6.1
Ef[^\̌î݁xWFAȉ~ցj

2003/08/17-20 ver 1.3.0
Ef[^̏ki̓_Ȃj

2003/08/05 FreeNote 1.1.1preJ
ENɕԂێ
E`惂[hؑւɏS\
EۑԒZk
EView[h̋ύX
Ej[̌ڂύX
*/
#include "fnpolygon.h"
#include <qmessagebox.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qimage.h>
#include <qfontdatabase.h>
#include <qpen.h>

const int FONTSIZE[9] = {
	10,
	12,
	16,
	18,
	20,
	24,
	36,
	48,
	72,
};

QColor SelectionLineColor = QColor(150, 150, 150);
QColor SelectionBrushColor = QColor(100, 100, 100);
QColor SelectionFrameColor = QColor(150, 150, 150);
QColor RulerColor = QColor(200, 200, 200);
QColor GuideColor = QColor(0,255,255);
QColor GridColor = QColor(50, 240, 240);

//FNPolygon
FNPolygon::FNPolygon() : _pen(Qt::black, 1), _points(0), _type(FN_POLYGON), _fill(false)
{
}

FNPolygon::FNPolygon(QPen& p) : _pen(p), _points(0), _type(FN_POLYGON), _fill(false)
{
}

FNPolygon::FNPolygon(const FNPolygon& p)
{
	operator=(p);
}

FNPolygon& FNPolygon::operator=(const FNPolygon& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
		this->_fill = p._fill;
	}
	return *this;
}

FNPolygon::~FNPolygon()
{
}

void FNPolygon::setPoints(FNPointList& list)
{
	_points.resize(list.count());
	for (uint i = 0; i < list.count(); ++i) {
		QPoint* p = list.at(i);
		_points.setPoint(i, QPoint(*p));
	}
}

void FNPolygon::translate(int dx, int dy)
{
	_points.translate(dx, dy);
}

void FNPolygon::drawShape(QPainter& pa, bool selected)
{
	pa.setBrush(Qt::NoBrush);
	pa.setPen(Qt::NoPen);
	if (_fill) {
		if (selected) {
			pa.setBrush(SelectionBrushColor);
			pa.setPen(QPen(SelectionLineColor));
		} else {
			pa.setPen(_pen);
			pa.setBrush(_pen.color());
		}
		pa.drawPolygon(_points);
	} else {
		if (selected) {
			pa.setPen(QPen(SelectionLineColor, 1, Qt::DashDotLine));
		} else {
			pa.setPen(_pen);
		}
		pa.drawPolyline(_points);
	}
	pa.flush();
}

QRect FNPolygon::boundingRect() const
{
	return _points.boundingRect();
}

FNBezier::FNBezier() : FNPolygon()
{
	_type = FN_BEZIER;
}

FNBezier::FNBezier(QPen& p) : FNPolygon(p)
{
	_type = FN_BEZIER;
}

FNBezier::FNBezier(const FNBezier& p) : FNPolygon()
{
	operator=(p);
}

FNBezier& FNBezier::operator=(const FNBezier& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
		this->_fill = p._fill;
	}
	return *this;
}

FNBezier::~FNBezier()
{
}

void FNBezier::drawShape(QPainter& pa, bool selected)
{
	pa.setBrush(Qt::NoBrush);
	pa.setPen(Qt::NoPen);
	if (_fill) {
		if (selected) {
			pa.setBrush(SelectionBrushColor);
			pa.setPen(QPen(SelectionLineColor));
		} else {
			pa.setPen(_pen);
			pa.setBrush(_pen.color());
		}
		QPoint tp = _points.point(0);
		FNPointList buf;
		buf.setAutoDelete(true);
		for (uint i = 1; i < _points.count(); i += 3) {
			QPointArray p(4);
			p.setPoint(0, tp);
			p.setPoint(1, _points.point(i));
			p.setPoint(2, _points.point(i + 1));
			p.setPoint(3, _points.point(i + 2));
			QPointArray pt = p.quadBezier();
			for (uint j = 0; j < pt.count(); ++j) {
				buf.append(new QPoint(pt[j]));
			}
			tp = _points.point(i + 2);
		}
		QPointArray po(buf.count());
		for (uint i = 0; i < buf.count(); ++i) {
			po.setPoint(i, *buf.at(i));
		}
		pa.drawPolygon(po);
	} else {
		if (selected) {
			pa.setPen(QPen(QColor(SelectionLineColor), 1, Qt::DashDotLine));
		} else {
			pa.setPen(_pen);
		}
		QPoint tp = _points.point(0);
		for (uint i = 1; i < _points.count(); i += 3) {
			QPointArray p(4);
			p.setPoint(0, tp);
			p.setPoint(1, _points.point(i));
			p.setPoint(2, _points.point(i + 1));
			p.setPoint(3, _points.point(i + 2));
			pa.drawPolyline(p.quadBezier());
			tp = _points.point(i + 2);
		}
	}
	pa.flush();
}

FNEllipse::FNEllipse() : FNPolygon()
{
	_type = FN_ELLIPSE;
}

FNEllipse::FNEllipse(QPen& p) : FNPolygon(p)
{
	_type = FN_ELLIPSE;
}

FNEllipse::FNEllipse(const FNEllipse& p) : FNPolygon()
{
	operator=(p);
}

FNEllipse& FNEllipse::operator=(const FNEllipse& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
		this->_fill = p._fill;
	}
	return *this;
}

FNEllipse::~FNEllipse()
{
}

void FNEllipse::drawShape(QPainter& pa, bool selected)
{
	pa.setBrush(Qt::NoBrush);
	pa.setPen(Qt::NoPen);
	if (_fill) {
		if (selected) {
			pa.setBrush(SelectionBrushColor);
			pa.setPen(QPen(SelectionLineColor));
		} else {
			pa.setBrush(_pen.color());
		}
	} else {
		if (selected) {
			pa.setPen(QPen(SelectionLineColor, 1, Qt::DashDotLine));
		} else {
			pa.setPen(_pen);
		}
	}
	QPoint sp = _points.point(0);
	QPoint ep = _points.point(1);
	pa.drawEllipse(sp.x(), sp.y(), ep.x() - sp.x(), ep.y() - sp.y());
	pa.flush();
}

FNText::FNText() : FNPolygon()
{
	_type = FN_TEXT;
}

FNText::FNText(QPen& p) : FNPolygon(p)
{
	_type = FN_TEXT;
}

FNText::FNText(const FNText& p) : FNPolygon()
{
	operator=(p);
}

FNText& FNText::operator=(const FNText& p)
{
	if (this != &p) {
		this->_type = p._type;
		this->_pen = p._pen;
		this->_points = p._points.copy();
		this->lines = p.lines;
		this->_fill = p._fill;
	}
	return *this;
}

FNText::~FNText()
{
}

void FNText::drawShape(QPainter& pa, bool selected)
{
	pa.setBrush(Qt::NoBrush);
	static QFontDatabase fbase;
	QFont font = pa.font();
	uint fmax = fbase.pointSizes(font.family()).count() - 1;
	if (selected) {
		pa.setPen(QPen(SelectionLineColor));
	} else {
		pa.setPen(_pen);
	}
	QPoint sp = _points.point(0);
	//default font size checking...
	if (fmax < _pen.width()) {
		_pen.setWidth(fmax);
	}
	font.setPointSize(FONTSIZE[_pen.width()]);
	QFontMetrics fm(font);
	int h = fm.height();
	int x = sp.x();
	int y = sp.y();
	y += h + 1;
	pa.setFont(font);
	for (uint i = 0; i < lines.count(); ++i) {
		pa.drawText(x + 1, y, lines[i]);
		y = y + h;
	}
	pa.flush();
}


FNLayer::FNLayer():IsShow(true), _disp_offset(0)
{
	draws.setAutoDelete(true);
}

FNLayer::~FNLayer() {
	draws.clear();
}

FNLayer::FNLayer(FNLayer& v)
{
	operator=(v);
}

FNLayer& FNLayer::operator=(FNLayer& v)
{
	if (this != &v) {
		IsShow = v.IsShow;
		_disp_offset = v._disp_offset;
		Name = v.Name;
		draws.clear();
		v.draws.clone(draws);
	}
	return *this;
}

uint FNLayer::disp_offset()
{
	return _disp_offset;
}


void FNLayer::undo()
{
	++_disp_offset;
	if (_disp_offset > draws.count()) {
		_disp_offset = draws.count();
	}
}

void FNLayer::redo()
{
	if (0 < _disp_offset) {
		--_disp_offset;
	}
}

void FNLayer::redobuf_flush()
{
	for (uint i = 0; i < _disp_offset; ++i) {
		FNPolygon* p = draws.last();
		draws.remove(p);
	}
	_disp_offset = 0;
}

void FNLayer::modeChanged()
{
	redobuf_flush();
}

void FNPolygonList::clone(FNPolygonList& lst)
{
	for (uint i = 0; i < this->count(); ++i) {
		uint type = this->at(i)->type();
		if (FN_BEZIER == type) {
			FNBezier* p = (FNBezier*)this->at(i);
			lst.append(new FNBezier(*p));
		} else if (FN_ELLIPSE == type) {
			FNEllipse* p = (FNEllipse*)this->at(i);
			lst.append(new FNEllipse(*p));
		} else if (FN_TEXT == type) {
			FNText* p = (FNText*)this->at(i);
			lst.append(new FNText(*p));
		} else {
			//FN_POLYGON
			lst.append(new FNPolygon(*this->at(i)));
		}
	}
}

void FNPolygonList::copy(FNPolygonList& lst)
{
	for (uint i = 0; i < this->count(); ++i) {
		lst.append(this->at(i));
	}
}
